
ASM;


.equ Fuellzeichen	= ' '


; Im Interrupt genutzte und geaenderte Register:

.def tempi		= r16

.def COPY_L		= r17
.def COPY_H		= r18

.def PixZei_Cnt		= r19		; Pixel-Zeilen-Counter

.def Spalt_Cnt		= r20		; Spalten Counter fuer die Zeichen

.def Zeile_Cnt		= r21		; Zeilen Counter fuer die Zeichen

.def ZeiHoehe_Cnt	= r22		; Counter fuer die Zeichenhoehe



; Register X (R26,R27) enthaelt den Zeiger auf den BWS im RAM fuer die Interruptroutine
; Register Y (R28,R29) enthaelt den Zeiger auf den BWS im RAM fuer den Cursor (aktuelle Position)
; Register Z (R30,R31) enthaelt den Zeiger auf den Font im Flash (mit aktuellem Offset)

; Z: Pointer auf die Schriftart (ZL: ASCII Code, ZH Pixelzeile)



; PortA
;A0 - Frame    - A -> L
;A1 - Load     - A -> L
;A2 - CP       - A -> L
;A3 - Disp_off - A -> L
;A7 - LED_GR   - A -> L

.equ 	FRAME		=0			; FRAME -> FLM, First Line Marker (VSync)
.equ 	LOAD		=1			; CL1, Latch Puls (HSync)
.equ 	COPY		=2			; CL2, Pixelclock
.equ 	DISP_OFF	=3			; Ein/Ausschalten der LCD-Kontrastspannung (Ausgang)
.equ 	LED_GR		=7			; gruene LED


; PortB = D0..D3
; PB5=MOSI, PB6=MISO, PB6=SCK -> waehrend der Programmierung fuer SPI genutzt (im Betrieb frei)



;Werte
.equ Framerate		=50		; Refreshrate
.equ F_CPU		=20000000	; CPU Freq in MHz



; Interne Konstanten
.equ PixelZeilen	=200
.equ HFREQ		=Framerate*PixelZeilen     ; 10000
.equ TDIV		=(F_CPU/16/HFREQ)-1       ; = (1250000/10000)-1 = 125-1 = 124
.equ Zeichenhoehe	=8		; 8, 10, 12
.equ maxSpalten		=78		; maximale physikalische Spaltenzahl minus 2
					; 80-2 -> muss eine gerade Zahl sein
.equ maxSpalten_Half 	=maxSpalten/2
.equ maxZeilen		=(PixelZeilen/Zeichenhoehe)-1	; 200/8-1 -> 24






	;*** Bildbeginn ***


        push    R17  ; COPY_L
        push    R18  ; COPY_H

        push    R19  ; PixZei_Cnt
        lds	PixZei_Cnt, Wert_PixZei_Cnt

        push    R20  ; Spalt_Cnt

        push    R21  ; Zeile_Cnt
        lds	Zeile_Cnt, Wert_Zeile_Cnt

        push    R22  ; ZeiHoehe_Cnt
        lds	ZeiHoehe_Cnt, Wert_ZeiHoehe_Cnt


        push    xl
        push    xh
        lds	xl, Wert_BWS_L
        lds	xh, Wert_BWS_H


        push    zl
        push    zh
        lds	zh, Wert_zh




	; vorbereiten der Register
	in 	tempi, PORTA
	; COPY = 0
	cbr 	tempi, 1<<COPY
	mov 	COPY_L, tempi
	; und COPY = 1
	sbr 	tempi, 1<<COPY
	mov 	COPY_H, tempi




	; Inkrementiern des Pixel-Zeilen-Counters
	inc	PixZei_Cnt


	; ueberspringen der ersten 4 Pixelzeilen
	cpi	PixZei_Cnt, 5		; 1..4 - 5 = Curry
	brcc	hloop_a

	rjmp	hloop_k


hloop_a:

	; ueberspringen der letzten 4 Pixelzeilen
	ldi 	R16, PixelZeilen-4
	cp	R16, PixZei_Cnt		; 196 - (197..200) = Curry
	brcc	hloop_b

	rjmp	hloop_k


hloop_b:

	; Lade Spalt_Cnt mit maxSpalten
	ldi 	Spalt_Cnt, maxSpalten


	; Ein Byte am Zeilenanfang mit Leerzeichen anzeigen
	ldi	R16, 0xFF

	out	PORTA, COPY_L
	nop
	nop
	out	PORTA, COPY_H
	out	PORTB, R16
	out	PORTA, COPY_L
	nop
	nop
	out	PORTA, COPY_H
	out	PORTB, R16



	; Loop fuer 1 Zeile
hloop_c:
	out 	PORTA, COPY_L		; PCLK L		1	-> 4 Takte H
	ld 	ZL, X+	                ; ASCII aus RAM lesen	2
	lpm	R16, Z	                ; Byte aus CGROM lesen	3
	out 	PORTA, COPY_H		; PCLK H		1	-> 4 Takte L
	out	PORTB, R16	        ; Low Nibble ausgeben	1
	out 	PORTA, COPY_L		; PCLK L		1	-> 2 Takte H
	swap 	R16			; 			1
	out 	PORTB, R16		; High Nibble ausgeben	1
	out 	PORTA, COPY_H		; PCLK H		1	-> 3 Takte L
	dec 	Spalt_Cnt	        ;			1
	brne 	hloop_c			;			2
					;		        15 Takte pro Byte


	; Ein Byte am Zeilenende mit Leerzeichen anzeigen
	ldi	R16, 0xFF

	out	PORTA, COPY_L
	nop
	nop
	out	PORTA, COPY_H
	out	PORTB, R16
	out	PORTA, COPY_L
	nop
	nop
	out	PORTA, COPY_H
	out	PORTB, R16



	; COPY_L auf 0
	out 	PORTA, COPY_L			; PCLK L

	; Schalten von LOAD und FRAME
	sbi 	PORTA, LOAD
	dec 	ZeiHoehe_Cnt			; Zaehler fuer Zeichenhoehe erniedrigen
	nop
	nop
	cbi 	PORTA, LOAD
	cbi 	PORTA, FRAME
	breq 	skipcloop			; 1 Textzeile fertig ausgegeben
	inc 	ZH				; Nein -> Bilddatenadresse fuer nachste Pixelzeile laden und

	sbiw 	XL, maxSpalten_Half		; Textadresse wieder an Anfang der Textzeile setzen
	sbiw 	XL, maxSpalten_Half

	rjmp 	hloop_m



	; Alles fuer neue Textzeile einrichten
skipcloop:
	ldi 	ZH, 8				; 1024Bytes CGROM Offset
	ldi 	ZeiHoehe_Cnt, Zeichenhoehe	; Zeichenhoehen Counter laden
	inc 	Zeile_Cnt			; Zeilencounter erhoehen
	cpi 	Zeile_Cnt, maxZeilen	        ; Bildende

        rjmp 	hloop_m



; *** Ausgabe einer Leer-Pixel-Zeile ***
;
hloop_k:
	; Lade Spalt_Cnt mit maxSpalten
	ldi 	Spalt_Cnt, maxSpalten+2


	; Ein Byte am Zeilenanfang mit Leerzeichen anzeigen
	ldi	R16, 0xFF

hloop_l:
	out	PORTA, COPY_L

	nop
	nop
        nop
        nop

	out	PORTA, COPY_H
	out	PORTB, R16
	out	PORTA, COPY_L

	nop
	nop

	out	PORTA, COPY_H
	out	PORTB, R16
	;
	dec	Spalt_Cnt
	brne 	hloop_l

	; COPY_L auf 0
	out 	PORTA, COPY_L		; PCLK L


	; Schalten von LOAD und FRAME
	sbi 	PORTA, LOAD
	nop
	nop
	cbi 	PORTA, LOAD
	cbi 	PORTA, FRAME


	; ist die letzte Bildschirmzeile geschrieben
	cpi	PixZei_Cnt, PixelZeilen
	brne	hloop_m


	; Grundeinstellungen
	clr	PixZei_Cnt
	clr	Zeile_Cnt
	ldi	XL, low(BWS)
	ldi	XH, high(BWS)

	; setzen von FRAME
	sbi 	PORTA, FRAME

hloop_m:

        sts     Wert_zh, zh
        pop     zh  ; font_h
        pop     zl  ; font_l

        sts     Wert_BWS_L, xl
        sts     Wert_BWS_H, xh
        pop     xh  ; Wert_BWS_H
        pop     xl  ; Wert_BWS_L

        sts	Wert_ZeiHoehe_Cnt, ZeiHoehe_Cnt
        pop     R22  ; ZeiHoehe_Cnt

        sts	Wert_Zeile_Cnt, Zeile_Cnt
        pop     R21  ; Zeile_Cnt

        pop     R20  ; Spalt_Cnt

        sts	Wert_PixZei_Cnt, PixZei_Cnt
        pop     R19  ; PixZei_Cnt

        pop     R18  ; COPY_H
        pop     R17  ; COPY_L


ENDASM;






