PROGRAM LCDM;

DEVICE = MEGA644P;

{$I mega644P_Register.INC}

{$I mega644P_TC.INC}

CONST


  // Frequenz = 20.000 MHz
  UBRR_VAL : WORD = 32;  // 38400 Baud
//  UBRR_VAL : WORD = 21;  // 57600 Baud
//  UBRR_VAL : WORD = 10;  // 115200 Baud    


  // Frequenz = 20.000.000 Hz;
  //CTC_WERT = ((F_CPU/16/HFREQ)-1  ; = 125-1 = 124
//  CTC_WERT : Word = 124;
  CTC_WERT : Word = 150;

  Laenge_BWS : Word = 1871;  // 1..1872 oder 0..1871
  Anzahl_Spalten : Word = 78;


VAR

{
A0 - Frame    - A -> L
A1 - Load     - A -> L
A2 - CP       - A -> L
A3 - Disp_off - A -> L
A7 - LED_GR   - A -> L
}
  Frame[PortA,0]    : bit;
  Load[PortA,1]     : bit;
  CP[PortA,2]       : bit;
  Disp_off[PortA,3] : bit;
  LED_GR[PortA, 7]  : bit;

  ZE_Zaehler, SP_Zaehler,
  Ausgabe : Byte;

  BWS : Array [0..1871] of Byte;

  Fuellbyte : Byte;
  BWS_Pos : Word;

  Wert_PixZei_Cnt,
  Wert_Zeile_Cnt,
  Wert_ZeiHoehe_Cnt,
  Wert_BWS_H, Wert_BWS_L,
  Wert_zh : Byte;

  ESC_Sequenz_folgt : Boolean;

  Write_String : String[20];




// *** INITIALISIERUNG ***


procedure InitPorts;
begin

{
A0 - Frame    - A -> L
A1 - Load     - A -> L
A2 - CP       - A -> L
A3 - Disp_off - A -> L
A7 - LED_GR   - A -> L
}
  //       AEEEAAAA
  DDRA:=  %10001111;

  // PORT B Voreinstellungen
  PORTA:= %00000000;



{
PB0..PB3 = Daten = D0..D3
}
  //       EEEEAAAA
  DDRB:=  %00001111;

  // PORT B Voreinstellungen
  PortB:= %00000000;

end InitPorts;





isr TIMER1_COMPA;
begin

{$I ANZEIGE.INC}

end TIMER1_COMPA;









// ******************************************
//
//       Hilfsprogramme fuer BWS
//
// ******************************************




procedure Schreibe_Zeichen_in_BWS(das_Byte : Byte);
begin
  BWS[BWS_Pos]:=das_Byte;

  if BWS_Pos<Laenge_BWS then
    Inc(BWS_Pos);
  endif;

end Schreibe_Zeichen_in_BWS;






procedure Screen_Roll_Up;
var
  w1 : Word;
  w2 : Word;
  w3 : Word;
  b1 : Byte;
begin

  w2:=Anzahl_Spalten;
  w3:=Laenge_BWS-Anzahl_Spalten;

  for w1:=0 to w3 do
    b1:=BWS[w2];
    BWS[w1]:=b1;
    Inc(w2);
  endfor;

  Inc(w3);

  for w1:=w3+1 to Laenge_BWS do
    BWS[w1]:=Fuellbyte;
  endfor;

end Screen_Roll_Up;





procedure berechne_LF;
var
  w1 : Word;
begin

  w1:=Laenge_BWS-Anzahl_Spalten;
  w1:=w1+1;

  if BWS_Pos<w1 then
    BWS_Pos:=BWS_Pos+Anzahl_Spalten;
  else
    Screen_Roll_Up;
    BWS_Pos:=w1;
  endif;

end berechne_LF;





procedure loesche_BWS;
var
  Zeichen_Zaehler : Word;
begin

  for Zeichen_Zaehler:=0 to Laenge_BWS do
    BWS[Zeichen_Zaehler]:=Fuellbyte;
  endfor; { for Zeichen_Zaehler:=0 to Laenge_BWS do }

  BWS_Pos:=0;

end loesche_BWS;





procedure berechne_CR;
var
  w1 : Word;
begin

  w1:=BWS_Pos div Anzahl_Spalten;
  BWS_Pos:=w1 * Anzahl_Spalten;

end berechne_CR;






procedure Schreibe_Byte_in_BWS(das_Byte : Byte);
begin

  if ESC_Sequenz_folgt then



  else

    case das_Byte of

      $0A : berechne_LF;
       |

      $0C : loesche_BWS;
       |

      $0D : berechne_CR;
       |

      $1B : ESC_Sequenz_folgt:=true;
       |

      $20..$7F : Schreibe_Zeichen_in_BWS(das_Byte);
       |

    else
      Schreibe_Zeichen_in_BWS($3C);
      Schreibe_Zeichen_in_BWS(das_Byte);
      Schreibe_Zeichen_in_BWS($3E);
    endcase; // case das_Byte of


  endif; // if ESC_Sequenz_folgt then

end Schreibe_Byte_in_BWS;






procedure fuelle_BWS_mit_Zeichen;
var
  Zeichen_Zaehler : Word;
  das_Zeichen : Byte;
begin
  das_Zeichen:=0;
  BWS_Pos:=0;

  for Zeichen_Zaehler:=0 to Laenge_BWS do

    Schreibe_Zeichen_in_BWS(das_Zeichen);

    Inc(das_Zeichen);

  endfor; { for Zeichen_Zaehler:=0 to Laenge_BWS do }

end fuelle_BWS_mit_Zeichen;









procedure Write;
var
  c1 : Char;
  Laenge : Byte;
  b1 : Byte;
  b2 : Byte;
begin

  Laenge:=Length(Write_String);
  for b1:=1 to Laenge do
    c1:=Write_String[b1];
    b2:=CharToByte(c1);
    Schreibe_Byte_in_BWS(b2);
  endfor;

end Write;





procedure WriteLn;
begin
  Write;
  Schreibe_Byte_in_BWS($0D); // CR
  Schreibe_Byte_in_BWS($0A); // LF
end WriteLn;












BEGIN


ASM;

.equ UBRR0 = UBRR0L

ENDASM;

  Fuellbyte:=$2E;    // .

  InitPorts;

  Init_TIMER1_COMPA(CTC_WERT);



  ClearBit(Frame);
  ClearBit(Load);
  ClearBit(CP);
  ClearBit(Disp_off);
  ClearBit(LED_GR);



  Wert_PixZei_Cnt:=0;
  Wert_Zeile_Cnt:=0;
  Wert_ZeiHoehe_Cnt:=8;

  ESC_Sequenz_folgt:=false;

  loesche_BWS;

ASM;

   ldi    R16, low(BWS)
   sts    Wert_BWS_L, R16

   ldi    R16, high(BWS)
   sts    Wert_BWS_H, R16

   ldi    R16, 8       		; 1024Bytes CGROM Offset
   sts    Wert_zh, R16

ENDASM;

  loesche_BWS;

  EnableInts;

  // nach dem Interrupt wird das LCD eingeschaltet
  SetBit(Disp_off);
  SetBit(LED_GR);

  loop


    Write_String:='Hallo Welt ???';
    WriteLn;
    Write_String:='Dies ist ein';
    Write;
    Write_String:=' kleiner Text.';
    WriteLn;

    Write_String:='Hallo Welt !!!';
    WriteLn;
    Write_String:='Dies ist ein';
    Write;
    Write_String:=' kleiner Text.';
    WriteLn;


  endloop;


{$I FONT8X8.INC}


END LCDM.
